package cz.muni.fi.pa165.core.rest;

import cz.muni.fi.pa165.core.api.StewardApiDelegate;
import cz.muni.fi.pa165.core.model.PageStewardDto;
import cz.muni.fi.pa165.core.model.PageableObject;
import cz.muni.fi.pa165.core.model.SortObject;
import cz.muni.fi.pa165.core.model.StewardDto;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

@RestController
public class StewardController implements StewardApiDelegate {

    private final List<StewardDto> stewardDtos = new CopyOnWriteArrayList<>();

    @Override
    public ResponseEntity<List<StewardDto>> getAllStewards() {
        stewardDtos.add(new StewardDto().id(1L).firstName("John").lastName("Doe"));
        return new ResponseEntity<>(stewardDtos, HttpStatus.OK);
    }

    @Override
    public ResponseEntity<StewardDto> getSteward(Long id) {
        var stewardDto = stewardDtos
                .stream()
                .filter(x -> x.getId().equals(id))
                .findFirst()
                .orElse(null);

        return new ResponseEntity<>(stewardDto, stewardDto != null ? HttpStatus.OK : HttpStatus.NOT_FOUND);
    }

    @Override
    public ResponseEntity<PageStewardDto> getStewardsPaged(Integer page, Integer size, List<String> sort) {
        var pageRequest = PageRequest.of(page, size);
        var stewards = stewardDtos.stream().skip(pageRequest.getOffset()).limit(pageRequest.getPageSize()).toList();
        var result = new PageImpl<>(stewards, pageRequest, stewardDtos.size());

        var s = new SortObject()
                .sorted(pageRequest.getSort().isSorted())
                .unsorted(pageRequest.getSort().isUnsorted())
                .empty(pageRequest.getSort().isEmpty());

        var pageableObject = new PageableObject()
                .paged(pageRequest.isPaged())
                .unpaged(pageRequest.isUnpaged())
                .pageNumber(pageRequest.getPageNumber())
                .pageSize(pageRequest.getPageSize())
                .sort(s)
                .offset(pageRequest.getOffset());

        var pageStewardDto = new PageStewardDto()
                .content(stewards)
                .pageable(pageableObject)
                .last(result.isLast())
                .first(result.isFirst())
                .empty(result.isEmpty())
                .totalPages(result.getTotalPages())
                .totalElements(result.getTotalElements())
                .number(result.getNumber())
                .numberOfElements(result.getNumberOfElements())
                .size(result.getSize())
                .sort(s);

        return new ResponseEntity<>(pageStewardDto, HttpStatus.OK);
    }
}
